#!/usr/bin/perl

# Drakwizard

# Copyright (C) 2003 Florent Villard <warly@mandrakesoft.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

package MDK::Wizard::Bind;
use lib qw(/usr/lib/libDrakX);
use strict;

use common;
use MDK::Wizard::Wizcommon;

my $wiz = new MDK::Wizard::Wizcommon;
my $dns1;
my $dns2;

my $o = { 
    name => 'DNS Configuration Wizard',
	var => { 
	    ext_dns1 => '',
	    ext_dns2 => ''
	},
	needed_rpm => [ 'bind' ],
	defaultimage => "/usr/share/wizards/dns_wizard/images/DNS.png"
    };

$o->{pages} = { 
		  welcome => {
		      name => N('DNS configuration wizard') . "\n\n" . N('DNS (Domain Name Server) is the service that maps an IP address of a machine with an internet host name.') . "\n\n" . N('This wizard will help you configuring the DNS services of your server. This configuration will provide a local DNS service for local computers names, with non-local requests forwarded to an outside DNS.'),
		  no_back => 1,
	          next => 'dns_ip'
	      },
	      dns_ip => {
	          name => N('DNS Server Addresses') . "\n\n" . N('DNS will allow your network to communicate with the Internet using standard internet host names.  In order to configure DNS, you must provide the IP address of primary and secondary DNS server; usually this address are given by your Internet provider.') . N('IP addresses are a dotted list of four numbers smaller than 256'), 
                  pre => sub {
		      $o->{var}{ext_dns1} ||= $dns1;
		      $o->{var}{ext_dns2} ||= $dns2
		  },
                  data => [
		      { label => N('Primary DNS Address'),  val => \$o->{var}{ext_dns1} },
		      { label => N('Secondary DNS Address:'), val => \$o->{var}{ext_dns2} },
		      ],
	          next => 'summary'
	      },
	      ip_error => {
		name => 'Error.' . "\n\n" . N('This is not a valid address... press next to continue'),
		ignore => 1,
	        next => 'dns_ip'
	      },
	      ip_warning => {
		name => 'Warning.' . "\n\n" .  N('You have entered an empty address for the DNS server.') . "\n\n" .
		      N('Your setting could be accepted, but you will not be able to identify machine names outside your local network.') . "\n\n" .
		      N('"Press next to leave these values empty, or back to enter a value.'),
	        next => 'summary'
	      },
	      summary => {
	        name => N('Configuring the DNS Server') . "\n\n" . N('The wizard collected the following parameters needed to configure your DNS service:') . "\n\n" . N('To accept these values, and configure your server, click the Next button or use the Back button to correct them'),
                data => [
		      { label => N('Primary DNS Address:'), fixed_val => \$o->{var}{ext_dns1} },
		      { label => N('Secondary DNS Address:'), fixed_val => \$o->{var}{ext_dns2} },
		  ],
		post => \&do_it,
	        next => 'end'
	      },
	      end => { 
	        name => N('Congratulations'),
                data => [ { label => N('The wizard successfully configured the DNS services of your server.') } ], 
		end => 1,
	        next => 0
	  },
};

sub up_serial {
    my	($file) = @_;

    my (undef, undef, undef, $mday, $mon, $year) = gmtime(time);
    $year += 1900;
    my	$serial_nbm = sprintf "%4dY%2dm%2d00", $year, $mon, $mday;
    output($file, map { 
	my $line = $_;
	if (/^(\s*)(\d*)(\s*;\s*Serial.*)$/) {
	    my $serial_f = $2;
	    $serial_f++;
	    if ($serial_f <= $serial_nbm) {
		$serial_f = $serial_nbm;
		chomp($serial_f);
		$line = "$1$serial_f$3\n";
	    }
	}
	$line;
    } cat_($file));
}

sub do_it {
    $::testing and return;
    my $wiz_ip_server = $wiz->{net}->itf_get("IPADDR");
    my $wiz_domain_name = $wiz->{net}->network_get("DOMAINNAME"); 
    my $wiz_host_name = $wiz->{net}->network_get("HOSTNAME"); 
    my $s_trunc = "$1.$2.$3" if $wiz_ip_server =~ /(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})/;
    my $ds = "$4" if $wiz_ip_server =~ /(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})/;
    my $host = "$1" if $wiz_host_name =~ /(.*?)\..*/;
    my $reversnet = "$3.$2.$1" if $wiz_ip_server =~ /(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})/;
    my $file = "/etc/host.conf";

    -f $file and MDK::Common::cp_af($file, $file.".orig");
    MDK::Common::cp_af("/usr/share/wizards//dns_wizard/scripts/host.conf.default", $file);

    $file = "/etc/named.conf";
    -f $file and MDK::Common::cp_af($file, $file.".orig");

    my $ispns1 = $o->{var}{ext_dns1} || "// __ISPN1__";
    my $ispns2 = $o->{var}{ext_dns2} || "// __ISPN2__";

    output($file, map { 
	s|__ISPNS1__|$ispns1|g;
	s|__ISPNS2__|$ispns2|g;
	s|__dname__|$wiz_domain_name|g;
	s|__revnet__|$reversnet|g;
	s|__net__|$s_trunc|g;
	$_;
    } cat_("/usr/share/wizards//dns_wizard/scripts/named.conf.default"));

    standalone::explanations("$file : DNS1: $ispns1 DNS2: $ispns2");
# Bug fix for bind 9:
    if (! -f "/etc/rndc.key") { system("touch /etc/rndc.key") or die "can not touch /etc/rndc.key"};

# root.hints
    $file="/var/named/root.hints";
    -f $file and MDK::Common::cp_af($file, $file . ".orig");
	MDK::Common::cp_af("/usr/share/wizards//dns_wizard/scripts/root.hints.default", $file);
	standalone::explanations("$file clobbered");
# 127.0.0.rev
    $file="/var/named/127.0.0.rev";
    -f $file and MDK::Common::cp_af($file, $file . ".orig");
    output($file, map {
	s|__hname__|$wiz_host_name|g;
	$_;
    } cat_("/usr/share/wizards//dns_wizard/scripts/127.0.0.rev.default"));
    standalone::explanations("$file : hostname: $wiz_host_name");
    up_serial($file);

# $ipnet.rev
    $file = "/var/named/$s_trunc.rev";
    -f $file and MDK::Common::cp_af($file, $file.".orig");
    output($file, map {
	s|__dname__|$wiz_domain_name|g;
	s|__hname__|$wiz_host_name|g;
	s|__revnet__|$reversnet|g;
	s|__nb__|$ds|g;
	$_;
    } cat_("/usr/share/wizards//dns_wizard/scripts/ipnet.rev.default"));
    up_serial($file);
    standalone::explanations("$file : domain: $wiz_domain_name hostname: $wiz_host_name revnet: $reversnet
nb: $ds");
# $domain.db
    $file = "/var/named/$wiz_domain_name.db";
    -f $file and MDK::Common::cp_af($file, $file.".orig");
    output($file, map {
	s|__dname__|$wiz_domain_name|g;
	s|__hname__|$wiz_host_name|g;
	s|__ip__|$wiz_ip_server|g;
	s|__host__|$host|g;
	s|__nb__|$ds|g;
	$_;
    } cat_("/usr/share/wizards//dns_wizard/scripts/domain.db.default"));
    up_serial($file);
    standalone::explanations("$file : domain: $wiz_domain_name hostname: $wiz_host_name revnet: $reversnet
nb: $ds");
# resolv.conf
    $file = "/etc/resolv.conf";
    -f $file and MDK::Common::cp_af($file, $file.".orig");
    open(NEW, "> $file");
    print NEW "domain $wiz_domain_name\n";
    print NEW "nameserver $wiz_ip_server\n";
    standalone::explanations("$file : domain: $wiz_domain_name nameserver: $wiz_ip_server");
    system("/sbin/chkconfig --level 235 named on");
    system("/etc/rc.d/init.d/named restart");
}

sub new {
    my ($class, $conf) = @_;
    my $file = "/etc/named.conf";
    if (-f $file) {
	open my $NEW, $file;
	DNS : while (<$NEW>) {
	    $dns2 = "";
	    if (m/^\s*options\s*\{/s...m/^\s*\}\;/s ) {
		if (m/^\s*forwarders\s*\{/s...m/^\s*\}\;/s ) {
		    if (/(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})/) {
			$dns2 = "$1.$2.$3.$4";
			$dns1 and next DNS;
			$dns1 = $dns2;
		    }
		}
	    }
	}
	close $NEW;
    }
    if (!$dns1) {
	$dns1 = $wiz->{net}->network_get("dnsServer");
	if ($dns1 eq '127.0.0.1') { 
	    $dns1 = $wiz->{net}->network_get("dnsServer2");
	    $dns2 = $wiz->{net}->network_get("dnsServer3") || '127.0.0.1'
	} else {
	    $dns2 = $wiz->{net}->network_get("dnsServer2")
	}
    }

    bless {
	    o   => $o,
       }, $class;
}

1;
